
% this program is the formulation of the adaptive SSA for artifacts removal
% and rhythms extraction of EEG signal.

clc;clear;close all

% import the EEG data V_noise
V_noise = xlsread('open5.xlsx');
% V_noise = xlsread('close3.xlsx');

%% the parameters in the program
fs=200;            % the sampling rate fs = 200Hz
L = 40;            % window length of embedding L = 40
comp=9;            % the number of matrices in the SSA
N=length(V_noise); % number of samples or the length of the data
time=(0:N-1)/fs;   % the time vector 
dmax = max(abs(V_noise)); % the amplitude of the EEG signal

%% the main program
% embeding step in the SSA to map the trajectory matrix X
[ X ] = func_embedding( V_noise, L);

% SVD step in the SSA to obtain the eigenvalues D_s and elementary matrices Xi
[D_s,Xi] = func_svd( X,comp);

% the reconstruction stage to obtain the reconstructed components RC
[ RC ] = func_reconstruction (Xi, N);

% artifact removalto obtian the artifacts: artifact_ssa
[artifact_ssa ]=func_artifactsremoval(RC,dmax);
sig_ssa = V_noise - artifact_ssa; % corrected EEG

% alpha rhythm extraction 
[alpha_ssa] = func_rhythmsextraction(RC,D_s,dmax,fs);

% extracted alpha rhythm using wavelet decomposition
[alpha_wp] = func_waveletpacketdecomposite(V_noise);

% extracted alpha rhythm using IIR filtering
[alpha_IIR] = func_IIRfiltering(V_noise,fs);

%% Power and Power ratio of the extracted alpha rhythm
% Power and Power ratio using adaptive SSA
power=(alpha_ssa.^2);
power_total=(sig_ssa.^2);
P=sum(power)/N               % power 
P_total=sum(power_total)/N;
P_rate=P/P_total             % power ratio

% Power using wavelet decomposition
power_wp=(alpha_wp.^2);
P_wp=sum(power_wp)/N

% Power  using IIR filtering
power_IIR=(alpha_IIR.^2);
P_IIR=sum(power_IIR)/N

%% figures
% artifacts removal
figure 
subplot(3,1,1)
plot(time,V_noise)
title('Contaminated EEG');
ylabel('uV')
xlabel('Time (s)')
subplot(3,1,2)
plot(time,sig_ssa)
title('Corrected EEG');
ylabel('uV')
xlabel('Time (s)')
ylim([-100 100])
subplot(3,1,3)
plot(time,artifact_ssa)
title('Removed artifacts');
ylabel('uV')
xlabel('Time (s)')

% alpha rhythm extraction 
figure 
subplot(2,1,1)
plot(time,alpha_ssa)
title('Extracted alpha rhythm');
ylabel('uV')
xlabel('Time (s)')
% power spectrum of extracted alpha rhythm 
Nfft=2048; 
window=kaiser(512,3);   
noverlap=256;  
[Pssa,fp]=pwelch(alpha_ssa,window,noverlap,Nfft,fs);   
subplot(2,1,2)
plot(fp,Pssa);
xlim([0 30])
xlabel('Frequencies (Hz)');ylabel('Power Spectrum (uV^2/Hz)');
title('Power Spectrum of Extracted Alpha Rhythm'); grid on
% spectrogram of the extracted alpha rhythm 
A = zeros(256,1);
data = [A' alpha_ssa' A'];
[b,f,t]=spectrogram(data,512,412,512,200);
figure
imagesc(t-1.28,f,abs(b)); axis xy, colormap(jet);  
ylim([5 15])
ylabel('Frequency (Hz)')
xlabel('Time (s)')
zlim([0 100])



