% this program is the formulation of the adaptive SSA for artifacts removal
% and rhythms extraction of simulated EEG signal 

clc;clear;close all

%% the parameters in the program
fs=200;            % the sampling rate fs = 200Hz
L = 40;            % window length of embedding L = 40
comp=9;            % the number of matrices in the SSA
EOG = 100;         % amplitude of the EOG artifact in the EEG simulation

%% the main program
% simulate the EEG based on the MPA model
[V_noise V_alpha time] = func_EEGgenerationMPAmodel(EOG,fs);

N=length(V_noise); % number of samples or the length of the data
dmax = max(abs(V_noise)); % the amplitude of the EEG signal

% embeding step in the SSA to map the trajectory matrix X
[ X ] = func_embedding( V_noise, L);

% SVD step in the SSA to obtain the eigenvalues D_s and elementary matrices Xi
[D_s,Xi] = func_svd( X,comp);

% the reconstruction stage to obtain the reconstructed components RC
[ RC ] = func_reconstruction (Xi, N);

% artifact removalto obtian the artifacts: artifact_ssa
[artifact_ssa ]=func_artifactsremoval(RC,dmax);
sig_ssa = V_noise' - artifact_ssa; % corrected EEG

% alpha rhythm extraction 
[alpha_ssa] = func_rhythmsextraction(RC,D_s,dmax,fs);

%% figures
% artifacts removal 
figure
subplot(3,1,1)
plot(time,V_noise)
% ylim([-200 400])
title('EEG with Aritfacts');
ylabel('uV')
subplot(3,1,2)
plot(time,sig_ssa)
% ylim([-500 500])
ylim([-200 200])
title('Corrected EEG');
ylabel('uV')
subplot(3,1,3)
plot(time,artifact_ssa)
% ylim([-200 400])
ylim([-200 200])
title('Extracted Artifacts');
ylabel('uV')
xlabel('Time (s)')

% alpha rhythm extracton
figure
subplot(2,1,1)
plot(time,V_alpha)
title('Simulated Alpha Rhythm');
ylim([-50 50])
xlabel('Time (s)')
ylabel('uV')
subplot(2,1,2)
plot(time,alpha_ssa)
title('Extracted Alpha Rhythm Using Adaptive SSA');
ylim([-50 50])
ylabel('uV')
xlabel('Time (s)')

% power spectrum of the simulated and extracted alpha rhythms
Nfft=2048; 
window=kaiser(512,3);  
noverlap=256;   
[Pxx,fp]=pwelch(V_alpha,window,noverlap,Nfft,fs);
[Pssa,fp]=pwelch(alpha_ssa,window,noverlap,Nfft,fs);
figure    
plot(fp,Pxx,fp,Pssa,'-r');  
xlim([0 30])
xlabel('Frequency (Hz)');ylabel('Power Spectrum (uV^2/Hz)');
title('Power Spectrum of Alpha Rhythm'); grid on
legend('Simulated Alpha','Extracted Alpha');





